import csv
from collections import defaultdict
from openpyxl import Workbook


def wczytaj_dane(plik_csv):
    with open(plik_csv, newline='', encoding='utf-8') as csvfile:
        reader = csv.DictReader(csvfile)
        naglowki = reader.fieldnames
        dane = list(reader)
    return naglowki, dane


def wykryj_typy_pytan(naglowki, dane):
    jednokrotne = []
    wielokrotne = []
    for naglowek in naglowki:
        if any(";" in (wiersz[naglowek] or "") for wiersz in dane):
            wielokrotne.append(naglowek)
        else:
            jednokrotne.append(naglowek)
    return jednokrotne, wielokrotne


def generuj_tabele(dane, key1, key2):
    tabela = defaultdict(lambda: defaultdict(int))
    liczba_osob = defaultdict(int)
    odpowiedzi_unikalne = set()

    for wiersz in dane:
        grupa = (wiersz.get(key1) or "").strip()
        odpowiedzi = set((wiersz.get(key2) or "").split(";"))
        odpowiedzi = {o.strip() for o in odpowiedzi if o.strip()}
        if not grupa:
            continue
        liczba_osob[grupa] += 1
        for odp in odpowiedzi:
            tabela[grupa][odp] += 1
            odpowiedzi_unikalne.add(odp)

    return tabela, liczba_osob, sorted(odpowiedzi_unikalne)


def zapisz_do_excel(tabele, plik_wyjsciowy):
    wb = Workbook()
    wb.remove(wb.active)

    for key1, lista_tabel in tabele.items():
        ws = wb.create_sheet(title=key1[:31].replace('/', '-'))
        row = 1
        for key2, (tabela, liczba_osob, odpowiedzi) in lista_tabel:
            # Dodaj nazwę pytania key2 nad tabelą
            ws.cell(row=row, column=1, value=f"Pytanie: {key2}")
            row += 1
            
            # Nagłówek
            ws.cell(row=row, column=1, value=key1)
            for col_idx, odp in enumerate(odpowiedzi, start=2):
                ws.cell(row=row, column=col_idx, value=odp)
            row += 1

            # Dane
            for grupa in sorted(tabela):
                ws.cell(row=row, column=1, value=grupa)
                for col_idx, odp in enumerate(odpowiedzi, start=2):
                    liczba = tabela[grupa].get(odp, 0)
                    suma = liczba_osob[grupa]
                    procent = (liczba / suma * 100) if suma else 0
                    ws.cell(row=row, column=col_idx, value=round(procent, 1))
                row += 1

            # Pusty wiersz
            row += 1

    wb.save(plik_wyjsciowy)


def main():
    NAZWA_PLIKU = "Kwestionariusz.csv"
    PLIK_WYJSCIOWY = "giga_tabelka.xlsx"

    naglowki, dane = wczytaj_dane(NAZWA_PLIKU)
    jednokrotne, wielokrotne = wykryj_typy_pytan(naglowki, dane)

    tabele = defaultdict(list)

    for key1 in jednokrotne:
        for key2 in wielokrotne:
            tabela, liczba_osob, odpowiedzi = generuj_tabele(dane, key1, key2)
            if odpowiedzi:  # Pomijamy puste
                tabele[key1].append((key2, (tabela, liczba_osob, odpowiedzi)))

    zapisz_do_excel(tabele, PLIK_WYJSCIOWY)
    print(f"✅ Zapisano wszystkie tabele do pliku: {PLIK_WYJSCIOWY}")


if __name__ == "__main__":
    main()
